"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSeedFactories = exports.createClassSeeders = exports.createClassSeeder = exports.ClassSeeder = void 0;
const faker_1 = require("@faker-js/faker");
const common_1 = require("@nestjs/common");
const references_1 = require("../relationships/references");
const seeder_factory_1 = require("../seeder/seeder.factory");
const util_1 = require("util");
class ClassSeeder {
    constructor(seedClass, options) {
        this.seedClass = seedClass;
        this.options = options;
        this.factories = getSeedFactories(seedClass);
        this.logger = new common_1.Logger(`ClassSeeder:${this.getName()}`);
        this.logger.debug(`Registered factory properties: ${Object.values(this.factories)
            .map((f) => f.propertyKey)
            .join(", ")}`);
    }
    getName() {
        return this.seedClass.name;
    }
    async seed(context) {
        this.logger.debug(`Start of seeding ...`);
        const ctx = context;
        const batchSize = await this.getBatchSize();
        const repo = ctx.dataSource.getRepository(this.seedClass);
        const records = await this.createRecords(batchSize, ctx);
        const refs = pilferReferences(records);
        ctx.unresolvedReferences = ctx.unresolvedReferences.concat(refs);
        this.logger.debug(`Saving records ...`);
        let saved = await repo.save(records.map(r => (Object.assign({}, r))));
        ctx.savedEntities.set(this.seedClass, saved);
        this.logger.debug(`Saved.`);
        const resolvePromises = [];
        const counter = ctx.unresolvedReferences.length;
        ctx.unresolvedReferences = ctx.unresolvedReferences.filter((ref) => {
            if (ref.refClass === this.seedClass)
                resolvePromises.push(ref.resolve(saved));
            else
                return true;
        });
        this.logger.debug(`Resolving ${counter - ctx.unresolvedReferences.length} references ...`);
        if (resolvePromises.length) {
            await Promise.all(resolvePromises);
            saved = await repo.save(records);
            ctx.savedEntities.set(this.seedClass, saved);
            this.logger.debug(`Resolved.`);
        }
        this.logger.debug(`Finished.`);
        return saved;
    }
    async drop(ctx) {
        const repo = ctx.dataSource.getRepository(this.seedClass);
        await repo.delete({});
    }
    async getBatchSize() {
        return this.options.count || 10;
    }
    async createRecords(n, context) {
        this.logger.debug(`Generating ${n} records.`);
        const ctx = context;
        ctx.currentClass = this.seedClass;
        ctx.currentBatchSize = n;
        ctx.currentBatchRecords = [];
        ctx.currentRecords.set(this.seedClass, ctx.currentBatchRecords);
        const records = Array(n);
        for (let i = 0; i < records.length; i++) {
            ctx.currentIndex = i;
            records[i] = ctx.previousRecord = await this.createRecord(context);
            ctx.currentBatchRecords.push(ctx.previousRecord);
        }
        ctx.currentClass = undefined;
        this.logger.debug(`Generated records: ${(0, util_1.inspect)(records, { depth: 2, colors: true })}`);
        return records;
    }
    async createRecord(context) {
        const ctx = context;
        const instance = (ctx.currentRecord = {});
        for (const factory of Object.values(this.factories)) {
            instance[factory.propertyKey] = await factory.generate(faker_1.faker, context);
        }
        return instance;
    }
}
exports.ClassSeeder = ClassSeeder;
function createClassSeeder(seedClass, options = {}) {
    const productClass = options.seeder || (ClassSeeder);
    new common_1.Logger("ClassSeederFactory").debug(`Creating ${productClass.name} for ${seedClass.name}`);
    return {
        provide: Symbol(`Class seeder of '${seedClass.name}'`),
        useFactory: () => new productClass(seedClass, options),
    };
}
exports.createClassSeeder = createClassSeeder;
function createClassSeeders(seedClasses, options = {}) {
    let optionsArray = Array.isArray(options)
        ? options
        : Array.from({ length: seedClasses.length }, () => options);
    return seedClasses.map((seedClass, i) => createClassSeeder(seedClass, optionsArray[i]));
}
exports.createClassSeeders = createClassSeeders;
function getSeedFactories(seedClass) {
    return Reflect.getMetadataKeys(seedClass)
        .map((key) => Reflect.getMetadata(key, seedClass))
        .filter((meta) => meta instanceof seeder_factory_1.SeederFactory)
        .reduce((acc, factory) => {
        acc[factory.propertyKey] = factory;
        return acc;
    }, {});
}
exports.getSeedFactories = getSeedFactories;
function pilferReferences(records) {
    const refs = [];
    for (const record of records) {
        for (const [prop, value] of Object.entries(record)) {
            if (value instanceof references_1.Ref) {
                value.propertyKey = prop;
                refs.push(value);
                delete record[prop];
            }
        }
    }
    return refs;
}
